@@ -30,6 +30,7 @@ module Network.TLS
 	-- * High level API
 	, sendData
 	, recvData
+	, recvData'
 
 	-- * Crypto Key
 	, PrivateKey(..)
@@ -44,6 +45,9 @@ module Network.TLS
 	, Version(..)
 	-- * Errors
 	, TLSError(..)
+	-- * Exceptions
+	, HandshakeFailed(..)
+	, ConnectionNotEstablished(..)
 	) where
 
 import Network.TLS.Struct (Version(..), TLSError(..))
