@@ -1,4 +1,5 @@
 {-# OPTIONS_HADDOCK hide #-}
+{-# LANGUAGE DeriveDataTypeable #-}
 -- |
 -- Module      : Network.TLS.Core
 -- License     : BSD-style
@@ -21,10 +22,13 @@ module Network.TLS.Core
 	-- * Initialisation and Termination of context
 	, bye
 	, handshake
+	, HandshakeFailed(..)
+	, ConnectionNotEstablished(..)
 
 	-- * High level API
 	, sendData
 	, recvData
+	, recvData'
 	) where
 
 import Network.TLS.Context
@@ -39,6 +43,7 @@ import Network.TLS.Receiving
 import Network.TLS.Measurement
 import Network.TLS.Wire (encodeWord16)
 import Data.Maybe
+import Data.Data
 import Data.List (intersect, find)
 import qualified Data.ByteString as B
 import qualified Data.ByteString.Lazy as L
@@ -51,10 +56,29 @@ import System.IO (Handle)
 import System.IO.Error (mkIOError, eofErrorType)
 import Prelude hiding (catch)
 
+data HandshakeFailed = HandshakeFailed TLSError
+	deriving (Show,Eq,Typeable)
+
+data ConnectionNotEstablished = ConnectionNotEstablished
+	deriving (Show,Eq,Typeable)
+
+instance Exception HandshakeFailed
+instance Exception ConnectionNotEstablished
+
 errorToAlert :: TLSError -> Packet
 errorToAlert (Error_Protocol (_, _, ad)) = Alert [(AlertLevel_Fatal, ad)]
 errorToAlert _                           = Alert [(AlertLevel_Fatal, InternalError)]
 
+handshakeFailed :: TLSError -> IO ()
+handshakeFailed err = throwIO $ HandshakeFailed err
+
+checkValid :: MonadIO m => TLSCtx c -> m ()
+checkValid ctx = do
+	established <- ctxEstablished ctx
+	unless established $ liftIO $ throwIO ConnectionNotEstablished
+	eofed <- ctxEOF ctx
+	when eofed $ liftIO $ throwIO $ mkIOError eofErrorType "data" Nothing Nothing
+
 readExact :: MonadIO m => TLSCtx c -> Int -> m Bytes
 readExact ctx sz = do
 	hdrbs <- liftIO $ connectionRecv ctx sz
@@ -149,14 +173,25 @@ sendPacket ctx pkt = do
 	liftIO $ connectionSend ctx dataToSend
 
 -- | Create a new Client context with a configuration, a RNG, a generic connection and the connection operation.
-clientWith :: (MonadIO m, CryptoRandomGen g) => TLSParams -> g -> c -> IO () -> (Bytes -> IO ()) -> (Int -> IO Bytes) -> m (TLSCtx c)
+clientWith :: (MonadIO m, CryptoRandomGen g)
+           => TLSParams         -- ^ Parameters to use for this context
+           -> g                 -- ^ Random number generator associated
+           -> c                 -- ^ An abstract connection type
+           -> IO ()             -- ^ A method for the connection buffer to be flushed
+           -> (Bytes -> IO ())  -- ^ A method for sending bytes through the connection
+           -> (Int -> IO Bytes) -- ^ A method for receiving bytes through the connection
+           -> m (TLSCtx c)
 clientWith params rng connection flushF sendF recvF =
 	liftIO $ newCtxWith connection flushF sendF recvF params st
 	where st = (newTLSState rng) { stClientContext = True }
 
 -- | Create a new Client context with a configuration, a RNG, and a Handle.
 -- It reconfigures the handle buffermode to noBuffering
-client :: (MonadIO m, CryptoRandomGen g) => TLSParams -> g -> Handle -> m (TLSCtx Handle)
+client :: (MonadIO m, CryptoRandomGen g)
+       => TLSParams -- ^ parameters to use for this context
+       -> g         -- ^ random number generator associated with the context
+       -> Handle    -- ^ handle to use
+       -> m (TLSCtx Handle)
 client params rng handle = liftIO $ newCtx handle params st
 	where st = (newTLSState rng) { stClientContext = True }
 
@@ -193,6 +228,8 @@ handshakeTerminate ctx = do
 	-- forget all handshake data now and reset bytes counters.
 	usingState_ ctx endHandshake
 	updateMeasure ctx resetBytesCounters
+	-- mark the secure connection up and running.
+	setEstablished ctx True
 	return ()
 
 -- client part of handshake. send a bunch of handshake of client
@@ -441,38 +478,34 @@ handshakeServer ctx = do
 
 -- | Handshake for a new TLS connection
 -- This is to be called at the beginning of a connection, and during renegociation
-handshake :: MonadIO m => TLSCtx c -> m Bool
+handshake :: MonadIO m => TLSCtx c -> m ()
 handshake ctx = do
 	cc <- usingState_ ctx (stClientContext <$> get)
 	liftIO $ handleException $ if cc then handshakeClient ctx else handshakeServer ctx
 	where
-		handleException f = catch (f >> return True) (\e -> handler e >> return False)
-		handler e = case fromException e of
-			Just err -> sendPacket ctx (errorToAlert err)
-			Nothing  -> sendPacket ctx (errorToAlert $ Error_Misc $ show e)
+		handleException f = catch f $ \exception -> do
+			let tlserror = maybe (Error_Misc $ show exception) id $ fromException exception
+			setEstablished ctx False
+			sendPacket ctx (errorToAlert tlserror)
+			handshakeFailed tlserror
 
 -- | sendData sends a bunch of data.
 -- It will automatically chunk data to acceptable packet size
 sendData :: MonadIO m => TLSCtx c -> L.ByteString -> m ()
-sendData ctx dataToSend = do
-	eofed <- ctxEOF ctx
-	when eofed $ liftIO $ throwIO $ mkIOError eofErrorType "sendData" Nothing Nothing
-	mapM_ sendDataChunk (L.toChunks dataToSend)
-		where sendDataChunk d = if B.length d > 16384
-			then do
-				let (sending, remain) = B.splitAt 16384 d
-				sendPacket ctx $ AppData sending
-				sendDataChunk remain
-			else
-				sendPacket ctx $ AppData d
+sendData ctx dataToSend = checkValid ctx >> mapM_ sendDataChunk (L.toChunks dataToSend)
+	where sendDataChunk d
+		| B.length d > 16384 = do
+			let (sending, remain) = B.splitAt 16384 d
+			sendPacket ctx $ AppData sending
+			sendDataChunk remain
+		| otherwise = sendPacket ctx $ AppData d
 
 -- | recvData get data out of Data packet, and automatically renegociate if
 -- a Handshake ClientHello is received
-recvData :: MonadIO m => TLSCtx c -> m L.ByteString
+recvData :: MonadIO m => TLSCtx c -> m Bytes
 recvData ctx = do
-	eofed <- ctxEOF ctx
-	when eofed $ liftIO $ throwIO $ mkIOError eofErrorType "recvData" Nothing Nothing
-	pkt   <- recvPacket ctx
+	checkValid ctx
+	pkt <- recvPacket ctx
 	case pkt of
 		-- on server context receiving a client hello == renegociation
 		Right (Handshake [ch@(ClientHello _ _ _ _ _ _)]) ->
@@ -482,10 +515,13 @@ recvData ctx = do
 			handshakeClient ctx >> recvData ctx
 		Right (Alert [(AlertLevel_Fatal, _)]) -> do
 			setEOF ctx
-			return L.empty
+			return B.empty
 		Right (Alert [(AlertLevel_Warning, CloseNotify)]) -> do
 			setEOF ctx
-			return L.empty
-		Right (AppData x) -> return $ L.fromChunks [x]
+			return B.empty
+		Right (AppData x) -> return x
 		Right p           -> error ("error unexpected packet: " ++ show p)
 		Left err          -> error ("error received: " ++ show err)
+
+recvData' :: MonadIO m => TLSCtx c -> m L.ByteString
+recvData' ctx = recvData ctx >>= return . L.fromChunks . (:[])
