@@ -16,20 +16,17 @@ module Network.TLS.State
 	, TLSHandshakeState(..)
 	, TLSCryptState(..)
 	, TLSMacState(..)
-	, TLSStatus(..)
-	, HandshakeStatus(..)
 	, newTLSState
 	, genTLSRandom
 	, withTLSRNG
 	, withCompression
 	, assert -- FIXME move somewhere else (Internal.hs ?)
-	, updateStatusHs
-	, updateStatusCC
 	, updateVerifiedData
 	, finishHandshakeTypeMaterial
 	, finishHandshakeMaterial
 	, makeDigest
 	, setMasterSecret
+	, setMasterSecretFromPre
 	, setPublicKey
 	, setPrivateKey
 	, setKeyBlock
@@ -39,19 +36,21 @@ module Network.TLS.State
 	, setSecureRenegotiation
 	, getSecureRenegotiation
 	, getVerifiedData
+	, setSession
+	, getSession
+	, getSessionData
+	, isSessionResuming
 	, switchTxEncryption
 	, switchRxEncryption
 	, getCipherKeyExchangeType
 	, isClientContext
 	, startHandshakeClient
 	, updateHandshakeDigest
-	, updateHandshakeDigestSplitted
 	, getHandshakeDigest
 	, endHandshake
 	) where
 
 import Data.Word
-import Data.List (find)
 import Data.Maybe (isNothing)
 import Network.TLS.Util
 import Network.TLS.Struct
@@ -72,28 +71,6 @@ assert :: Monad m => String -> [(String,Bool)] -> m ()
 assert fctname list = forM_ list $ \ (name, assumption) -> do
 	when assumption $ fail (fctname ++ ": assumption about " ++ name ++ " failed")
 
-data HandshakeStatus =
-	  HsStatusClientHello
-	| HsStatusServerHello
-	| HsStatusServerCertificate
-	| HsStatusServerKeyXchg
-	| HsStatusServerCertificateReq
-	| HsStatusServerHelloDone
-	| HsStatusClientCertificate
-	| HsStatusClientKeyXchg
-	| HsStatusClientCertificateVerify
-	| HsStatusClientChangeCipher
-	| HsStatusClientFinished
-	| HsStatusServerChangeCipher
-	deriving (Show,Eq)
-
-data TLSStatus =
-	  StatusInit
-	| StatusHandshakeReq
-	| StatusHandshake HandshakeStatus
-	| StatusOk
-	deriving (Show,Eq)
-
 data TLSCryptState = TLSCryptState
 	{ cstKey        :: !Bytes
 	, cstIV         :: !Bytes
@@ -122,8 +99,9 @@ instance Show StateRNG where
 data TLSState = TLSState
 	{ stClientContext       :: Bool
 	, stVersion             :: !Version
-	, stStatus              :: !TLSStatus
 	, stHandshake           :: !(Maybe TLSHandshakeState)
+	, stSession             :: Session
+	, stSessionResuming     :: Bool
 	, stTxEncrypted         :: Bool
 	, stRxEncrypted         :: Bool
 	, stTxCryptState        :: !(Maybe TLSCryptState)
@@ -155,8 +133,9 @@ newTLSState :: CryptoRandomGen g => g -> TLSState
 newTLSState rng = TLSState
 	{ stClientContext       = False
 	, stVersion             = TLS10
-	, stStatus              = StatusInit
 	, stHandshake           = Nothing
+	, stSession             = Session Nothing
+	, stSessionResuming     = False
 	, stTxEncrypted         = False
 	, stRxEncrypted         = False
 	, stTxCryptState        = Nothing
@@ -210,68 +189,6 @@ makeDigest w hdr content = do
 	modify (\_ -> if w then st { stTxMacState = Just newms } else st { stRxMacState = Just newms })
 	return digest
 
-hsStatusTransitionTable :: [ (HandshakeType, TLSStatus, [ TLSStatus ]) ]
-hsStatusTransitionTable =
-	[ (HandshakeType_HelloRequest, StatusHandshakeReq,
-		[ StatusOk ])
-	, (HandshakeType_ClientHello, StatusHandshake HsStatusClientHello,
-		[ StatusInit, StatusHandshakeReq ])
-	, (HandshakeType_ServerHello, StatusHandshake HsStatusServerHello,
-		[ StatusHandshake HsStatusClientHello ])
-	, (HandshakeType_Certificate, StatusHandshake HsStatusServerCertificate,
-		[ StatusHandshake HsStatusServerHello ])
-	, (HandshakeType_ServerKeyXchg, StatusHandshake HsStatusServerKeyXchg,
-		[ StatusHandshake HsStatusServerHello
-		, StatusHandshake HsStatusServerCertificate ])
-	, (HandshakeType_CertRequest, StatusHandshake HsStatusServerCertificateReq,
-		[ StatusHandshake HsStatusServerHello
-		, StatusHandshake HsStatusServerCertificate
-		, StatusHandshake HsStatusServerKeyXchg ])
-	, (HandshakeType_ServerHelloDone, StatusHandshake HsStatusServerHelloDone,
-		[ StatusHandshake HsStatusServerHello
-		, StatusHandshake HsStatusServerCertificate
-		, StatusHandshake HsStatusServerKeyXchg
-		, StatusHandshake HsStatusServerCertificateReq ])
-	, (HandshakeType_Certificate, StatusHandshake HsStatusClientCertificate,
-		[ StatusHandshake HsStatusServerHelloDone ])
-	, (HandshakeType_ClientKeyXchg, StatusHandshake HsStatusClientKeyXchg,
-		[ StatusHandshake HsStatusServerHelloDone
-		, StatusHandshake HsStatusClientCertificate ])
-	, (HandshakeType_CertVerify, StatusHandshake HsStatusClientCertificateVerify,
-		[ StatusHandshake HsStatusClientKeyXchg ])
-	, (HandshakeType_Finished, StatusHandshake HsStatusClientFinished,
-		[ StatusHandshake HsStatusClientChangeCipher ])
-	, (HandshakeType_Finished, StatusOk,
-		[ StatusHandshake HsStatusServerChangeCipher ])
-	]
-
-updateStatus :: MonadState TLSState m => TLSStatus -> m ()
-updateStatus x = modify (\st -> st { stStatus = x })
-
-updateStatusHs :: MonadState TLSState m => HandshakeType -> m (Maybe TLSError)
-updateStatusHs ty = do
-	status <- return . stStatus =<< get
-	ns <- return . transition . stStatus =<< get
-	case ns of
-		Nothing      -> return $ Just $ Error_Packet_unexpected (show status) ("handshake:" ++ show ty)
-		Just (_,x,_) -> updateStatus x >> return Nothing
-	where
-		edgeEq cur (ety, _, aprevs) = ty == ety && (maybe False (const True) $ find (== cur) aprevs)
-		transition currentStatus = find (edgeEq currentStatus) hsStatusTransitionTable
-
-updateStatusCC :: MonadState TLSState m => Bool -> m (Maybe TLSError)
-updateStatusCC sending = do
-	status <- return . stStatus =<< get
-	cc     <- isClientContext
-	let x = case (cc /= sending, status) of
-		(False, StatusHandshake HsStatusClientKeyXchg)           -> Just (StatusHandshake HsStatusClientChangeCipher)
-		(False, StatusHandshake HsStatusClientCertificateVerify) -> Just (StatusHandshake HsStatusClientChangeCipher)
-		(True, StatusHandshake HsStatusClientFinished)           -> Just (StatusHandshake HsStatusServerChangeCipher)
-		_                                                        -> Nothing
-	case x of
-		Just newstatus -> updateStatus newstatus >> return Nothing
-		Nothing        -> return $ Just $ Error_Packet_unexpected (show status) ("Client Context: " ++ show cc)
-
 updateVerifiedData :: MonadState TLSState m => Bool -> Bytes -> m ()
 updateVerifiedData sending bs = do
 	cc <- isClientContext
@@ -302,21 +219,52 @@ setServerRandom :: MonadState TLSState m => ServerRandom -> m ()
 setServerRandom ran = updateHandshake "srand" (\hst -> hst { hstServerRandom = Just ran })
 
 setMasterSecret :: MonadState TLSState m => Bytes -> m ()
-setMasterSecret premastersecret = do
-	st <- get
+setMasterSecret masterSecret = do
 	hasValidHandshake "master secret"
 
-	updateHandshake "master secret" (\hst ->
-		let ms = generateMasterSecret (stVersion st) premastersecret (hstClientRandom hst) (fromJust "server random" $ hstServerRandom hst) in
-		hst { hstMasterSecret = Just ms } )
+	updateHandshake "master secret" (\hst -> hst { hstMasterSecret = Just masterSecret } )
+	setKeyBlock
 	return ()
 
+setMasterSecretFromPre :: MonadState TLSState m => Bytes -> m ()
+setMasterSecretFromPre premasterSecret = do
+	hasValidHandshake "generate master secret"
+	st <- get
+	setMasterSecret $ genSecret st
+	where
+		genSecret st =
+			let hst = fromJust "handshake" $ stHandshake st in
+			generateMasterSecret (stVersion st)
+					     premasterSecret
+					     (hstClientRandom hst)
+					     (fromJust "server random" $ hstServerRandom hst)
+
 setPublicKey :: MonadState TLSState m => PublicKey -> m ()
 setPublicKey pk = updateHandshake "publickey" (\hst -> hst { hstRSAPublicKey = Just pk })
 
 setPrivateKey :: MonadState TLSState m => PrivateKey -> m ()
 setPrivateKey pk = updateHandshake "privatekey" (\hst -> hst { hstRSAPrivateKey = Just pk })
 
+getSessionData :: MonadState TLSState m => m (Maybe SessionData)
+getSessionData = do
+	st <- get
+	return (stHandshake st >>= hstMasterSecret >>= wrapSessionData st)
+	where wrapSessionData st masterSecret = do
+		return $ SessionData
+			{ sessionVersion = stVersion st
+			, sessionCipher  = cipherID $ fromJust "cipher" $ stCipher st
+			, sessionSecret  = masterSecret
+			}
+
+setSession :: MonadState TLSState m => Session -> Bool -> m ()
+setSession session resuming = modify (\st -> st { stSession = session, stSessionResuming = resuming })
+
+getSession :: MonadState TLSState m => m Session
+getSession = gets stSession
+
+isSessionResuming :: MonadState TLSState m => m Bool
+isSessionResuming = gets stSessionResuming
+
 setKeyBlock :: MonadState TLSState m => m ()
 setKeyBlock = do
 	st <- get
@@ -408,11 +356,6 @@ updateHandshakeDigest :: MonadState TLSState m => Bytes -> m ()
 updateHandshakeDigest content = updateHandshake "update digest" $ \hs ->
 	hs { hstHandshakeDigest = hashUpdate (hstHandshakeDigest hs) content }
 
-updateHandshakeDigestSplitted :: MonadState TLSState m => HandshakeType -> Bytes -> m ()
-updateHandshakeDigestSplitted ty bytes = updateHandshakeDigest $ B.concat [hdr, bytes]
-	where
-		hdr = runPut $ encodeHandshakeHeader ty (B.length bytes)
-
 getHandshakeDigest :: MonadState TLSState m => Bool -> m Bytes
 getHandshakeDigest client = do
 	st <- get
